/*
  Bewässerungs-System 1.0
  -----------------------------------------------------------
  Messure soil moisture of a plant (pot)
  If the soil is to dry, it will relaease a water pump cycle
  More functions:
  – Display background light turn of when not used (safe energy)
  – Error when sensor disconnected
  – Max. 5 water pump cycles until restart
  – Restart the system at the latest after one month – mills() overflow after approximately 50 day!

  Components:
  -----------------------------------------------------------
  Arduino UNO 
  Soil Moisture Sensor (FC-28)
  Relais 5V
  Water Pump 12V
  Display 1602
  Push-Button
  Potentiometer B10K

  Pin-Connections: 
  -----------------------------------------------------------
  Sensor:             A0
  Button:             7
  Display BG-Light:   8
  Relais:             9
  LCD:                12/11, 5/4/3/2

  modified 10.12.18
  by Chris Obrist

*/



// ----------------- Variables ------------

#include <LiquidCrystal.h>
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);

int moistureSensor1 = A0;                                  // define sensor input pin
int moistureValue1;                                        // define value for sensor

unsigned long previousMillis1 = 0;                         // will store last time was reading sensor (CHECK OVERFLOW)
const long interval1 = 5000;                               // interval between senor readings

unsigned long previousMillis2 = 0;                         // store last time was a water pumping cycle (CHECK OVERFLOW)
const long interval2 = 30000;                              // minimal pause between the water pump cycles

unsigned long previousMillis3 = 0;                         // store last time was a restart (CHECK OVERFLOW)
const long interval3 = 3600000;                            // restart the arduino after countdown

unsigned long previousMillis4 = 0;                         // store last time was dislplay on (CHECK OVERFLOW)
const long interval4 = 20000;                              // display off

const int button1 = 7;                                     // active when puss (HIGH), held low by 4.7K
int buttonValue1 = 0;                                      // pin HIGH/LOW status
int buttonState1 = 0;                                      // last button state

int relaisPin1 = 9;                                        // pin to connect the relais for water pump
int wateringTimeCycle = 5000;                              // duration of each watering cycle (1 sec. = 40 ml)
int cycleNumb1 = 0;                                        // counting the number of cycles
int cycleMax1 = 5;                                         // number of max cycles (until restart)

int displayLightPin = 8;


// ----------------- Setup ------------

void setup() {

  pinMode(button1, INPUT);                                  // define button pin
  pinMode(relaisPin1, OUTPUT);                              // defune relais pin
  pinMode(displayLightPin, OUTPUT);

  digitalWrite(relaisPin1, HIGH);                           // turn relais of at the restart (!)

  Serial.begin(9600);

  digitalWrite(displayLightPin, LOW);                       // display on (inverse logic!!!)
  
  lcd.begin(16, 2);                                         // 16 by 2 character display
  
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Hello, I'am your");                            // printing to the LCD display
  
  lcd.setCursor(0, 1);
  lcd.print("water system 1.0");                            // printing to the LCD display

  delay(3000);                                              // show message 3 sec. in display

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("I take care");                                 // printing to the LCD display
  
  lcd.setCursor(0, 1);
  lcd.print("of your plants!");                             // printing to the LCD display

  delay(3000);

}

void(* resetFunc) (void) = 0;                               // declare reset function at address 0

// ----------------- Loop ------------

void loop() {

  unsigned long currentMillis1 = millis();                  // timer for checking sensor
  unsigned long currentMillis2 = millis();                  // timer for pause between water pump cycles
  unsigned long currentMillis3 = millis();                  // timer for restart funktion
  unsigned long currentMillis4 = millis();                  // timer for display background light

  if (currentMillis1 - previousMillis1 >= interval1) {      // calculate interval for sensor
    previousMillis1 = currentMillis1;

    moistureValue1 = readMoistureSensor1();                 // messure moisture
  }


if (moistureValue1 < 50 && moistureValue1 > 10 && cycleNumb1 <= cycleMax1) {       // threshold for release water pump cycle in %
    
    if (currentMillis2 - previousMillis2 >= interval2) {    // calculate interval for water pump
    previousMillis2 = currentMillis2;
    digitalWrite(displayLightPin, LOW);                     // display on (inverse logic!!!)
    waterPumpCycle1();                                      // release water pump cycle function
    }
}
    
  if (moistureValue1 <= 10) {
  digitalWrite(displayLightPin, LOW);                     // display on (inverse logic!!!)
  
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Error1: Moisture");                          // printing to the LCD display

  lcd.setCursor(0, 1);
  lcd.print("Sensor disconnect");

  delay(5000);                                            // stay in error modus for 5 sec.
    
}


if (cycleNumb1 >= cycleMax1) {      
    digitalWrite(displayLightPin, LOW);                      // display on (inverse logic!!!)
    
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Error2: Stopping");                           // printing to the LCD display

    lcd.setCursor(0, 1);
    lcd.print("too much water!");

    delay(5000);                                             // stay in error modus for 5 sec.
}


  buttonValue1 = readButton1();                             // check button to release a water pump cycle manualy
// Serial.println(buttonValue1);                            // debugging button (write 0 or 1 in serial monitor)

  if (buttonValue1 == HIGH) {                               // if butten push release a manual water pump cycle
    digitalWrite(displayLightPin, LOW);                     // display on (inverse logic!!!) 
    waterPumpCycle1();                                      // release water pump cycle function
    
  }

  if (currentMillis3 - previousMillis3 >= interval3) {      // calculate interval for restart
    previousMillis3 = currentMillis3;
  digitalWrite(displayLightPin, LOW);                       // display on (inverse logic!!!)

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Time for restart");                            // printing to the LCD display
  
  lcd.setCursor(0, 1);
  lcd.print("bye, bye !!!");                                // printing to the LCD display

  delay(5000);                                              // show message 5 sec. in display

  resetFunc();                                              // call reset
}

  
  if (currentMillis4 - previousMillis4 >= interval4) {      // calculate interval for display background light
    previousMillis4 = currentMillis4;
    
  digitalWrite(displayLightPin, HIGH);                      // display off (inverse logic!!!)
  }

}

// ----------------- Funktions ------------

int readMoistureSensor1() {

  moistureValue1 = analogRead(moistureSensor1);             // read Moisture Sensor 1
  moistureValue1 = map(moistureValue1, 920, 200, 0, 100);   // calculate % – adjuste input values(!)

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Moisture: ");
  lcd.print(moistureValue1);                               // printing to the LCD display
  lcd.print("%");

  return (moistureValue1);                                 // give value of Sensor (%) back to the loop

}


int readButton1() {

  buttonValue1 = digitalRead(button1);

  if (buttonValue1 == HIGH && buttonState1 == LOW) {      // read button state and overwrite it
    buttonState1 = buttonValue1;
  }

  return (buttonValue1);                                  // give value of Sensor (1/0) back to the loop

}


int waterPumpCycle1() {

  digitalWrite(relaisPin1, LOW);                           // turn on relais (reverse logic!!!)
  cycleNumb1++;                                            // counting number of cycles (+1 each round)

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Waterpump1 on");                              // printing to the LCD display

  lcd.setCursor(0,1);
  lcd.print("Cycle number: ");                             // printing to the LCD display
  lcd.print(cycleNumb1);  
  
  delay(wateringTimeCycle);                                // watering time cycle
  digitalWrite(relaisPin1, HIGH);                          // turn off relais

}
